@@ -1,17 +1,21 @@
 import os
-import mock
 import pytest
 import re
 import redis
 import time
+from unittest import mock
 
 from threading import Thread
 from redis.connection import ssl_available, to_bool
-from .conftest import skip_if_server_version_lt, _get_client
+from .conftest import (
+    skip_if_server_version_lt,
+    skip_if_redis_enterprise,
+    _get_client
+)
 from .test_pubsub import wait_for_message
 
 
-class DummyConnection(object):
+class DummyConnection:
     description_format = "DummyConnection<>"
 
     def __init__(self, **kwargs):
@@ -25,7 +29,7 @@ class DummyConnection(object):
         return False
 
 
-class TestConnectionPool(object):
+class TestConnectionPool:
     def get_pool(self, connection_kwargs=None, max_connections=None,
                  connection_class=redis.Connection):
         connection_kwargs = connection_kwargs or {}
@@ -43,21 +47,25 @@ class TestConnectionPool(object):
         assert isinstance(connection, DummyConnection)
         assert connection.kwargs == connection_kwargs
 
-    def test_multiple_connections(self):
-        pool = self.get_pool()
+    def test_multiple_connections(self, master_host):
+        connection_kwargs = {'host': master_host[0], 'port': master_host[1]}
+        pool = self.get_pool(connection_kwargs=connection_kwargs)
         c1 = pool.get_connection('_')
         c2 = pool.get_connection('_')
         assert c1 != c2
 
-    def test_max_connections(self):
-        pool = self.get_pool(max_connections=2)
+    def test_max_connections(self, master_host):
+        connection_kwargs = {'host': master_host[0], 'port': master_host[1]}
+        pool = self.get_pool(max_connections=2,
+                             connection_kwargs=connection_kwargs)
         pool.get_connection('_')
         pool.get_connection('_')
         with pytest.raises(redis.ConnectionError):
             pool.get_connection('_')
 
-    def test_reuse_previously_released_connection(self):
-        pool = self.get_pool()
+    def test_reuse_previously_released_connection(self, master_host):
+        connection_kwargs = {'host': master_host[0], 'port': master_host[1]}
+        pool = self.get_pool(connection_kwargs=connection_kwargs)
         c1 = pool.get_connection('_')
         pool.release(c1)
         c2 = pool.get_connection('_')
@@ -89,7 +97,7 @@ class TestConnectionPool(object):
         assert repr(pool) == expected
 
 
-class TestBlockingConnectionPool(object):
+class TestBlockingConnectionPool:
     def get_pool(self, connection_kwargs=None, max_connections=10, timeout=20):
         connection_kwargs = connection_kwargs or {}
         pool = redis.BlockingConnectionPool(connection_class=DummyConnection,
@@ -98,22 +106,26 @@ class TestBlockingConnectionPool(object):
                                             **connection_kwargs)
         return pool
 
-    def test_connection_creation(self):
-        connection_kwargs = {'foo': 'bar', 'biz': 'baz'}
+    def test_connection_creation(self, master_host):
+        connection_kwargs = {'foo': 'bar', 'biz': 'baz',
+                             'host': master_host[0], 'port': master_host[1]}
         pool = self.get_pool(connection_kwargs=connection_kwargs)
         connection = pool.get_connection('_')
         assert isinstance(connection, DummyConnection)
         assert connection.kwargs == connection_kwargs
 
-    def test_multiple_connections(self):
-        pool = self.get_pool()
+    def test_multiple_connections(self, master_host):
+        connection_kwargs = {'host': master_host[0], 'port': master_host[1]}
+        pool = self.get_pool(connection_kwargs=connection_kwargs)
         c1 = pool.get_connection('_')
         c2 = pool.get_connection('_')
         assert c1 != c2
 
-    def test_connection_pool_blocks_until_timeout(self):
+    def test_connection_pool_blocks_until_timeout(self, master_host):
         "When out of connections, block for timeout seconds, then raise"
-        pool = self.get_pool(max_connections=1, timeout=0.1)
+        connection_kwargs = {'host': master_host[0], 'port': master_host[1]}
+        pool = self.get_pool(max_connections=1, timeout=0.1,
+                             connection_kwargs=connection_kwargs)
         pool.get_connection('_')
 
         start = time.time()
@@ -122,25 +134,28 @@ class TestBlockingConnectionPool(object):
         # we should have waited at least 0.1 seconds
         assert time.time() - start >= 0.1
 
-    def connection_pool_blocks_until_another_connection_released(self):
+    def test_connection_pool_blocks_until_conn_available(self, master_host):
         """
         When out of connections, block until another connection is released
         to the pool
         """
-        pool = self.get_pool(max_connections=1, timeout=2)
+        connection_kwargs = {'host': master_host[0], 'port': master_host[1]}
+        pool = self.get_pool(max_connections=1, timeout=2,
+                             connection_kwargs=connection_kwargs)
         c1 = pool.get_connection('_')
 
         def target():
             time.sleep(0.1)
             pool.release(c1)
 
-        Thread(target=target).start()
         start = time.time()
+        Thread(target=target).start()
         pool.get_connection('_')
         assert time.time() - start >= 0.1
 
-    def test_reuse_previously_released_connection(self):
-        pool = self.get_pool()
+    def test_reuse_previously_released_connection(self, master_host):
+        connection_kwargs = {'host': master_host[0], 'port': master_host[1]}
+        pool = self.get_pool(connection_kwargs=connection_kwargs)
         c1 = pool.get_connection('_')
         pool.release(c1)
         c2 = pool.get_connection('_')
@@ -150,7 +165,6 @@ class TestBlockingConnectionPool(object):
         pool = redis.ConnectionPool(
             host='localhost',
             port=6379,
-            db=0,
             client_name='test-client'
         )
         expected = ('ConnectionPool<Connection<'
@@ -161,7 +175,6 @@ class TestBlockingConnectionPool(object):
         pool = redis.ConnectionPool(
             connection_class=redis.UnixDomainSocketConnection,
             path='abc',
-            db=0,
             client_name='test-client'
         )
         expected = ('ConnectionPool<UnixDomainSocketConnection<'
@@ -169,39 +182,19 @@ class TestBlockingConnectionPool(object):
         assert repr(pool) == expected
 
 
-class TestConnectionPoolURLParsing(object):
-    def test_defaults(self):
-        pool = redis.ConnectionPool.from_url('redis://localhost')
-        assert pool.connection_class == redis.Connection
-        assert pool.connection_kwargs == {
-            'host': 'localhost',
-            'port': 6379,
-            'db': 0,
-            'username': None,
-            'password': None,
-        }
-
+class TestConnectionPoolURLParsing:
     def test_hostname(self):
-        pool = redis.ConnectionPool.from_url('redis://myhost')
+        pool = redis.ConnectionPool.from_url('redis://my.host')
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
-            'host': 'myhost',
-            'port': 6379,
-            'db': 0,
-            'username': None,
-            'password': None,
+            'host': 'my.host',
         }
 
     def test_quoted_hostname(self):
-        pool = redis.ConnectionPool.from_url('redis://my %2F host %2B%3D+',
-                                             decode_components=True)
+        pool = redis.ConnectionPool.from_url('redis://my %2F host %2B%3D+')
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'my / host +=+',
-            'port': 6379,
-            'db': 0,
-            'username': None,
-            'password': None,
         }
 
     def test_port(self):
@@ -210,35 +203,25 @@ class TestConnectionPoolURLParsing(object):
         assert pool.connection_kwargs == {
             'host': 'localhost',
             'port': 6380,
-            'db': 0,
-            'username': None,
-            'password': None,
         }
 
-    @skip_if_server_version_lt('5.9.101')
+    @skip_if_server_version_lt("6.0.0")
     def test_username(self):
         pool = redis.ConnectionPool.from_url('redis://myuser:@localhost')
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
-            'db': 0,
             'username': 'myuser',
-            'password': None,
         }
 
-    @skip_if_server_version_lt('5.9.101')
+    @skip_if_server_version_lt("6.0.0")
     def test_quoted_username(self):
         pool = redis.ConnectionPool.from_url(
-            'redis://%2Fmyuser%2F%2B name%3D%24+:@localhost',
-            decode_components=True)
+            'redis://%2Fmyuser%2F%2B name%3D%24+:@localhost')
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
-            'db': 0,
             'username': '/myuser/+ name=$+',
-            'password': None,
         }
 
     def test_password(self):
@@ -246,69 +229,51 @@ class TestConnectionPoolURLParsing(object):
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
-            'db': 0,
-            'username': None,
             'password': 'mypassword',
         }
 
     def test_quoted_password(self):
         pool = redis.ConnectionPool.from_url(
-            'redis://:%2Fmypass%2F%2B word%3D%24+@localhost',
-            decode_components=True)
+            'redis://:%2Fmypass%2F%2B word%3D%24+@localhost')
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
-            'db': 0,
-            'username': None,
             'password': '/mypass/+ word=$+',
         }
 
-    @skip_if_server_version_lt('5.9.101')
+    @skip_if_server_version_lt("6.0.0")
     def test_username_and_password(self):
         pool = redis.ConnectionPool.from_url('redis://myuser:mypass@localhost')
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
-            'db': 0,
             'username': 'myuser',
             'password': 'mypass',
         }
 
     def test_db_as_argument(self):
-        pool = redis.ConnectionPool.from_url('redis://localhost', db='1')
+        pool = redis.ConnectionPool.from_url('redis://localhost', db=1)
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
             'db': 1,
-            'username': None,
-            'password': None,
         }
 
     def test_db_in_path(self):
-        pool = redis.ConnectionPool.from_url('redis://localhost/2', db='1')
+        pool = redis.ConnectionPool.from_url('redis://localhost/2', db=1)
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
             'db': 2,
-            'username': None,
-            'password': None,
         }
 
     def test_db_in_querystring(self):
         pool = redis.ConnectionPool.from_url('redis://localhost/2?db=3',
-                                             db='1')
+                                             db=1)
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
             'db': 3,
-            'username': None,
-            'password': None,
         }
 
     def test_extra_typed_querystring_options(self):
@@ -320,13 +285,10 @@ class TestConnectionPoolURLParsing(object):
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
             'db': 2,
             'socket_timeout': 20.0,
             'socket_connect_timeout': 10.0,
             'retry_on_timeout': True,
-            'username': None,
-            'password': None,
         }
         assert pool.max_connections == 10
 
@@ -349,30 +311,17 @@ class TestConnectionPoolURLParsing(object):
         assert pool.connection_kwargs['client_name'] == 'test-client'
 
     def test_invalid_extra_typed_querystring_options(self):
-        import warnings
-        with warnings.catch_warnings(record=True) as warning_log:
+        with pytest.raises(ValueError):
             redis.ConnectionPool.from_url(
                 'redis://localhost/2?socket_timeout=_&'
                 'socket_connect_timeout=abc'
             )
-        # Compare the message values
-        assert [
-            str(m.message) for m in
-            sorted(warning_log, key=lambda l: str(l.message))
-        ] == [
-            'Invalid value for `socket_connect_timeout` in connection URL.',
-            'Invalid value for `socket_timeout` in connection URL.',
-        ]
 
     def test_extra_querystring_options(self):
         pool = redis.ConnectionPool.from_url('redis://localhost?a=1&b=2')
         assert pool.connection_class == redis.Connection
         assert pool.connection_kwargs == {
             'host': 'localhost',
-            'port': 6379,
-            'db': 0,
-            'username': None,
-            'password': None,
             'a': '1',
             'b': '2'
         }
@@ -386,50 +335,42 @@ class TestConnectionPoolURLParsing(object):
         assert r.connection_pool.connection_class == redis.Connection
         assert r.connection_pool.connection_kwargs == {
             'host': 'myhost',
-            'port': 6379,
-            'db': 0,
-            'username': None,
-            'password': None,
         }
 
     def test_invalid_scheme_raises_error(self):
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError) as cm:
             redis.ConnectionPool.from_url('localhost')
+        assert str(cm.value) == (
+            'Redis URL must specify one of the following schemes '
+            '(redis://, rediss://, unix://)'
+        )
 
 
-class TestConnectionPoolUnixSocketURLParsing(object):
+class TestConnectionPoolUnixSocketURLParsing:
     def test_defaults(self):
         pool = redis.ConnectionPool.from_url('unix:///socket')
         assert pool.connection_class == redis.UnixDomainSocketConnection
         assert pool.connection_kwargs == {
             'path': '/socket',
-            'db': 0,
-            'username': None,
-            'password': None,
         }
 
-    @skip_if_server_version_lt('5.9.101')
+    @skip_if_server_version_lt("6.0.0")
     def test_username(self):
         pool = redis.ConnectionPool.from_url('unix://myuser:@/socket')
         assert pool.connection_class == redis.UnixDomainSocketConnection
         assert pool.connection_kwargs == {
             'path': '/socket',
-            'db': 0,
             'username': 'myuser',
-            'password': None,
         }
 
-    @skip_if_server_version_lt('5.9.101')
+    @skip_if_server_version_lt("6.0.0")
     def test_quoted_username(self):
         pool = redis.ConnectionPool.from_url(
-            'unix://%2Fmyuser%2F%2B name%3D%24+:@/socket',
-            decode_components=True)
+            'unix://%2Fmyuser%2F%2B name%3D%24+:@/socket')
         assert pool.connection_class == redis.UnixDomainSocketConnection
         assert pool.connection_kwargs == {
             'path': '/socket',
-            'db': 0,
             'username': '/myuser/+ name=$+',
-            'password': None,
         }
 
     def test_password(self):
@@ -437,32 +378,24 @@ class TestConnectionPoolUnixSocketURLParsing(object):
         assert pool.connection_class == redis.UnixDomainSocketConnection
         assert pool.connection_kwargs == {
             'path': '/socket',
-            'db': 0,
-            'username': None,
             'password': 'mypassword',
         }
 
     def test_quoted_password(self):
         pool = redis.ConnectionPool.from_url(
-            'unix://:%2Fmypass%2F%2B word%3D%24+@/socket',
-            decode_components=True)
+            'unix://:%2Fmypass%2F%2B word%3D%24+@/socket')
         assert pool.connection_class == redis.UnixDomainSocketConnection
         assert pool.connection_kwargs == {
             'path': '/socket',
-            'db': 0,
-            'username': None,
             'password': '/mypass/+ word=$+',
         }
 
     def test_quoted_path(self):
         pool = redis.ConnectionPool.from_url(
-            'unix://:mypassword@/my%2Fpath%2Fto%2F..%2F+_%2B%3D%24ocket',
-            decode_components=True)
+            'unix://:mypassword@/my%2Fpath%2Fto%2F..%2F+_%2B%3D%24ocket')
         assert pool.connection_class == redis.UnixDomainSocketConnection
         assert pool.connection_kwargs == {
             'path': '/my/path/to/../+_+=$ocket',
-            'db': 0,
-            'username': None,
             'password': 'mypassword',
         }
 
@@ -472,8 +405,6 @@ class TestConnectionPoolUnixSocketURLParsing(object):
         assert pool.connection_kwargs == {
             'path': '/socket',
             'db': 1,
-            'username': None,
-            'password': None,
         }
 
     def test_db_in_querystring(self):
@@ -482,8 +413,6 @@ class TestConnectionPoolUnixSocketURLParsing(object):
         assert pool.connection_kwargs == {
             'path': '/socket',
             'db': 2,
-            'username': None,
-            'password': None,
         }
 
     def test_client_name_in_querystring(self):
@@ -497,28 +426,20 @@ class TestConnectionPoolUnixSocketURLParsing(object):
         assert pool.connection_class == redis.UnixDomainSocketConnection
         assert pool.connection_kwargs == {
             'path': '/socket',
-            'db': 0,
-            'username': None,
-            'password': None,
             'a': '1',
             'b': '2'
         }
 
 
-class TestSSLConnectionURLParsing(object):
-    @pytest.mark.skipif(not ssl_available, reason="SSL not installed")
-    def test_defaults(self):
-        pool = redis.ConnectionPool.from_url('rediss://localhost')
+@pytest.mark.skipif(not ssl_available, reason="SSL not installed")
+class TestSSLConnectionURLParsing:
+    def test_host(self):
+        pool = redis.ConnectionPool.from_url('rediss://my.host')
         assert pool.connection_class == redis.SSLConnection
         assert pool.connection_kwargs == {
-            'host': 'localhost',
-            'port': 6379,
-            'db': 0,
-            'username': None,
-            'password': None,
+            'host': 'my.host',
         }
 
-    @pytest.mark.skipif(not ssl_available, reason="SSL not installed")
     def test_cert_reqs_options(self):
         import ssl
 
@@ -547,7 +468,7 @@ class TestSSLConnectionURLParsing(object):
         assert pool.get_connection('_').check_hostname is True
 
 
-class TestConnection(object):
+class TestConnection:
     def test_on_connect_error(self):
         """
         An error in Connection.on_connect should disconnect from the server
@@ -564,6 +485,7 @@ class TestConnection(object):
         assert not pool._available_connections[0]._sock
 
     @skip_if_server_version_lt('2.8.8')
+    @skip_if_redis_enterprise
     def test_busy_loading_disconnects_socket(self, r):
         """
         If Redis raises a LOADING error, the connection should be
@@ -574,6 +496,7 @@ class TestConnection(object):
         assert not r.connection._sock
 
     @skip_if_server_version_lt('2.8.8')
+    @skip_if_redis_enterprise
     def test_busy_loading_from_pipeline_immediate_command(self, r):
         """
         BusyLoadingErrors should raise from Pipelines that execute a
@@ -589,6 +512,7 @@ class TestConnection(object):
         assert not pool._available_connections[0]._sock
 
     @skip_if_server_version_lt('2.8.8')
+    @skip_if_redis_enterprise
     def test_busy_loading_from_pipeline(self, r):
         """
         BusyLoadingErrors should be raised from a pipeline execution
@@ -604,6 +528,7 @@ class TestConnection(object):
         assert not pool._available_connections[0]._sock
 
     @skip_if_server_version_lt('2.8.8')
+    @skip_if_redis_enterprise
     def test_read_only_error(self, r):
         "READONLY errors get turned in ReadOnlyError exceptions"
         with pytest.raises(redis.ReadOnlyError):
@@ -629,6 +554,7 @@ class TestConnection(object):
             'path=/path/to/socket,db=0',
         )
 
+    @skip_if_redis_enterprise
     def test_connect_no_auth_supplied_when_required(self, r):
         """
         AuthenticationError should be raised when the server requires a
@@ -638,13 +564,14 @@ class TestConnection(object):
             r.execute_command('DEBUG', 'ERROR',
                               'ERR Client sent AUTH, but no password is set')
 
+    @skip_if_redis_enterprise
     def test_connect_invalid_password_supplied(self, r):
         "AuthenticationError should be raised when sending the wrong password"
         with pytest.raises(redis.AuthenticationError):
             r.execute_command('DEBUG', 'ERROR', 'ERR invalid password')
 
 
-class TestMultiConnectionClient(object):
+class TestMultiConnectionClient:
     @pytest.fixture()
     def r(self, request):
         return _get_client(redis.Redis,
@@ -657,7 +584,7 @@ class TestMultiConnectionClient(object):
         assert r.get('a') == b'123'
 
 
-class TestHealthCheck(object):
+class TestHealthCheck:
     interval = 60
 
     @pytest.fixture()
