@@ -1,7 +1,8 @@
-from __future__ import unicode_literals
 import pytest
 
-from redis3 import exceptions
+from redis import exceptions
+
+from tests.conftest import skip_if_server_version_lt
 
 
 multiply_script = """
@@ -21,7 +22,7 @@ return "hello " .. name
 """
 
 
-class TestScripting(object):
+class TestScripting:
     @pytest.fixture(autouse=True)
     def reset_scripts(self, r):
         r.script_flush()
@@ -31,6 +32,35 @@ class TestScripting(object):
         # 2 * 3 == 6
         assert r.eval(multiply_script, 1, 'a', 3) == 6
 
+    @skip_if_server_version_lt('6.2.0')
+    def test_script_flush_620(self, r):
+        r.set('a', 2)
+        r.script_load(multiply_script)
+        r.script_flush('ASYNC')
+
+        r.set('a', 2)
+        r.script_load(multiply_script)
+        r.script_flush('SYNC')
+
+        r.set('a', 2)
+        r.script_load(multiply_script)
+        r.script_flush()
+
+        with pytest.raises(exceptions.DataError):
+            r.set('a', 2)
+            r.script_load(multiply_script)
+            r.script_flush("NOTREAL")
+
+    def test_script_flush(self, r):
+        r.set('a', 2)
+        r.script_load(multiply_script)
+        r.script_flush(None)
+
+        with pytest.raises(exceptions.DataError):
+            r.set('a', 2)
+            r.script_load(multiply_script)
+            r.script_flush("NOTREAL")
+
     def test_evalsha(self, r):
         r.set('a', 2)
         sha = r.script_load(multiply_script)
